#! /usr/bin/python3

class Messages:
    fileNotFound = "{0}: Ce fichier ou ce dossier n'existe pas."
    fileAlreadyExists = "{0}: {1}: Le fichier existe déjà."
    blogName = "Nom du blog"
    yourName = "Votre nom"
    blogDescription = "Courte description du blog"
    blogKeywords = "Quelques mots clefs qui définissent votre blog"
    aboutYou = "À propos de vous"
    license = "La licence appliquée à votre contenu"
    blogUrl = "L'url du blog"
    blogLanguage = "La langue du blog"
    yourEmail = "Votre adresse email"
    missingParams = "{0}: Paramètres manquants."
    cannotReadIn = "{0}: Impossible de lire dans {1}."
    nothingToDo = "Rien à faire."
    unknownCommand = "{0}: Commande inconnue."
    noBlogConfiguration = "Le fichier de configuration du blog n'existe pas ou vous n'avez pas les bonnes permissions."
    missingMandatoryFieldInBlogConf = "Attention, le champ \"{0}\" est manquant dans le fichier de configuration principal du blog."
    doNotForgetToIncludeCSSFileInHeader= "N'oubliez pas d'inclure le fichier CSS {0} dans header.html."
    username="Nom d'utilisateur: "
    userPasswd="Mot de passe utilisateur: "
    ftpHost="Nom d'hôte FTP"
    ftpPath="Chemin absolu de votre blog sur l'hôte FTP."
    cleanFtpDirectory="Nettoyage du repertoire FTP de destination..."
    copyToFtpDirectory="Copie du blog vers le repertoire FTP de destination..."
    possibleMalformedEntry="La publication {0} est probablement mal formée... Abandon."
    blogRecompilation="Recompilation du blog locale..."
    exportArchives="Exportation locale du fil de publications de '{0}'."
    exportMainThread="Exportation locale du fil de publications principal."
    exportMainThreadRss="Exportation locale du flux RSS du fil de publications principal."
    exportCategories="Exportation locale du fil de publications de la categorie '{0}'."
    exportCategoriesRss="Exportation locale du flux RSS du fil de publications de la categorie '{0}'."
    missingMandatoryFieldInEntry="Le champ '{0}' dans l'entrée numéro {1} est manquant."
    getUnknownValue="Get: La valeur {0} n'existe pas."
    forUnknownValue="For: La valeur {0} n'existe pas."
    recursiveForUnknownValue="RecursiveFor: La valeur {0} n'existe pas."
    notEnoughArgs="Paramétres manquants."
    unknownPattern="Le motif '{0}' n'existe pas."
    unknownContextual="La variable contextuel {0} n'existe pas."
    inRessource="Dans la ressource '{0}'."
    somethingGoesWrongReturnEmptyString="Une erreur s'est produite. Remplacement pas une chaine de caractère vide."
    argBlogName="Nom du blog {0}"
    argEntryName="Nom de la publication"
    argTemplateName="Nom du template"
    argInputFilename="Nom de fichier"
    blogFolderDoesntExists="Le dossier de destination n'existe pas... VenC s'en fiche et va en faire un nouveau."
