#! /usr/bin/python3

class Messages:
    fileNotFound = "{0}: This file or directory does not exists."
    fileAlreadyExists = "{0}: {1}: File or directory already exists."
    blogName = "Blog name"
    yourName = "Your name"
    blogDescription = "Short description of your blog"
    blogKeywords = "Some keywords about your blog"
    aboutYou = "About you"
    license = "The license applied to your blog"
    blogUrl = "The URL of your blog"
    blogLanguage = "The language of your blog"
    yourEmail = "Your e-mail"
    missingParams = "{0}: Missing arguments."
    cannotReadIn = "{0}: Cannot read into {1}."
    nothingToDo = "Nothing to do."
    unknownCommand = "{0}: Unknown command."
    noBlogConfiguration = "Blog's configuration file doesn't exists or you do not have right permissions."
    missingMandatoryFieldInBlogConf = "Warning, the field \"{0}\" is missing the blog's configuration file."
    doNotForgetToIncludeCSSFileInHeader= "Do not forget to include CSS file {0} in header.html."
    username="Username: "
    userPasswd="User password: "
    ftpHost="FTP hostname"
    ftpPath="Absolute path of your blog on FTP server."
    cleanFtpDirectory="Cleaning target FTP directory..."
    copyToFtpDirectory="Copying your blog to target FTP directory..."
    possibleMalformedEntry="Possible malformed entry {0}. Abort."
    blogRecompilation="Recompilation of your blog locally..."
    exportArchives="Exporting local entries thread of '{0}'."
    exportMainThread="Exporting local main entries thread."
    exportMainThreadRss="Exporting local main RSS feed entries."
    exportCategories="Exporting local categories entries thread named '{0}'."
    exportCategoriesRss="Exporting local entries RSS feed thread from category '{0}'."
    missingMandatoryFieldInEntry="Field '{0}' in entry number {1} is missing."
    getUnknownValue="Get: Value {0} isn't defined."
    forUnknownValue="For: Value {0} isn't defined."
    recursiveForUnknownValue="RecursiveFor: Value {0} isn't defined."
    notEnoughArgs="Missing parameters."
    unknownPattern="The pattern '{0}'doesn't exists."
    unknownContextual="Contextual variable {0} doesn't exists."
    inRessource="In ressource '{0}'."
    somethingGoesWrongReturnEmptyString="An error occurs. Replacement with an empty string."
    argBlogName="Blog name {0}"
    argEntryName="Entry name"
    argTemplateName="Template name"
    argInputFilename="Filename"
    blogFolderDoesntExists="Destination folder doesn't exist. VenC doesn't care and will built a new one."
